@@ -2,14 +2,13 @@ import unittest
 import tkinter
 from tkinter import TclError
 import os
-import sys
 from test.support import requires
 
-from tkinter.test.support import (tcl_version, requires_tcl,
+from tkinter.test.support import (requires_tcl,
                                   get_tk_patchlevel, widget_eq,
                                   AbstractDefaultRootTest)
 from tkinter.test.widget_tests import (
-    add_standard_options, noconv, pixels_round,
+    add_standard_options,
     AbstractWidgetTest, StandardOptionsTests, IntegerSizeTests, PixelSizeTests,
     setUpModule)
 
@@ -21,7 +20,7 @@ def float_round(x):
 
 
 class AbstractToplevelTest(AbstractWidgetTest, PixelSizeTests):
-    _conv_pad_pixels = noconv
+    _conv_pad_pixels = False
 
     def test_configure_class(self):
         widget = self.create()
@@ -140,7 +139,7 @@ class LabelFrameTest(AbstractToplevelTest, unittest.TestCase):
 
 
 class AbstractLabelTest(AbstractWidgetTest, IntegerSizeTests):
-    _conv_pixels = noconv
+    _conv_pixels = False
 
     def test_configure_highlightthickness(self):
         widget = self.create()
@@ -250,7 +249,7 @@ class MenubuttonTest(AbstractLabelTest, unittest.TestCase):
         'takefocus', 'text', 'textvariable',
         'underline', 'width', 'wraplength',
     )
-    _conv_pixels = staticmethod(pixels_round)
+    _conv_pixels = round
 
     def create(self, **kwargs):
         return tkinter.Menubutton(self.root, **kwargs)
@@ -267,8 +266,6 @@ class MenubuttonTest(AbstractLabelTest, unittest.TestCase):
     test_configure_highlightthickness = \
         StandardOptionsTests.test_configure_highlightthickness
 
-    @unittest.skipIf(sys.platform == 'darwin',
-                     'crashes with Cocoa Tk (issue19733)')
     def test_configure_image(self):
         widget = self.create()
         image = tkinter.PhotoImage(master=self.root, name='image1')
@@ -348,10 +345,7 @@ class EntryTest(AbstractWidgetTest, unittest.TestCase):
         self.checkPixelsParam(widget, 'insertwidth', 1.3, 3.6, '10p')
         self.checkParam(widget, 'insertwidth', 0.1, expected=2)
         self.checkParam(widget, 'insertwidth', -2, expected=2)
-        if pixels_round(0.9) <= 0:
-            self.checkParam(widget, 'insertwidth', 0.9, expected=2)
-        else:
-            self.checkParam(widget, 'insertwidth', 0.9, expected=1)
+        self.checkParam(widget, 'insertwidth', 0.9, expected=1)
 
     def test_configure_invalidcommand(self):
         widget = self.create()
@@ -553,8 +547,6 @@ class TextTest(AbstractWidgetTest, unittest.TestCase):
         'tabs', 'tabstyle', 'takefocus', 'undo', 'width', 'wrap',
         'xscrollcommand', 'yscrollcommand',
     )
-    if tcl_version < (8, 5):
-        _stringify = True
 
     def create(self, **kwargs):
         return tkinter.Text(self.root, **kwargs)
@@ -563,12 +555,10 @@ class TextTest(AbstractWidgetTest, unittest.TestCase):
         widget = self.create()
         self.checkBooleanParam(widget, 'autoseparators')
 
-    @requires_tcl(8, 5)
     def test_configure_blockcursor(self):
         widget = self.create()
         self.checkBooleanParam(widget, 'blockcursor')
 
-    @requires_tcl(8, 5)
     def test_configure_endline(self):
         widget = self.create()
         text = '\n'.join('Line %d' for i in range(100))
@@ -592,7 +582,6 @@ class TextTest(AbstractWidgetTest, unittest.TestCase):
         widget = self.create()
         self.checkIntegerParam(widget, 'maxundo', 0, 5, -1)
 
-    @requires_tcl(8, 5)
     def test_configure_inactiveselectbackground(self):
         widget = self.create()
         self.checkColorParam(widget, 'inactiveselectbackground')
@@ -606,8 +595,7 @@ class TextTest(AbstractWidgetTest, unittest.TestCase):
     def test_configure_selectborderwidth(self):
         widget = self.create()
         self.checkPixelsParam(widget, 'selectborderwidth',
-                              1.3, 2.6, -2, '10p', conv=noconv,
-                              keep_orig=tcl_version >= (8, 5))
+                              1.3, 2.6, -2, '10p', conv=False)
 
     def test_configure_spacing1(self):
         widget = self.create()
@@ -624,7 +612,6 @@ class TextTest(AbstractWidgetTest, unittest.TestCase):
         self.checkPixelsParam(widget, 'spacing3', 20, 21.4, 22.6, '0.5c')
         self.checkParam(widget, 'spacing3', -10, expected=0)
 
-    @requires_tcl(8, 5)
     def test_configure_startline(self):
         widget = self.create()
         text = '\n'.join('Line %d' for i in range(100))
@@ -640,27 +627,18 @@ class TextTest(AbstractWidgetTest, unittest.TestCase):
 
     def test_configure_state(self):
         widget = self.create()
-        if tcl_version < (8, 5):
-            self.checkParams(widget, 'state', 'disabled', 'normal')
-        else:
-            self.checkEnumParam(widget, 'state', 'disabled', 'normal')
+        self.checkEnumParam(widget, 'state', 'disabled', 'normal')
 
     def test_configure_tabs(self):
         widget = self.create()
-        if get_tk_patchlevel() < (8, 5, 11):
-            self.checkParam(widget, 'tabs', (10.2, 20.7, '1i', '2i'),
-                            expected=('10.2', '20.7', '1i', '2i'))
-        else:
-            self.checkParam(widget, 'tabs', (10.2, 20.7, '1i', '2i'))
+        self.checkParam(widget, 'tabs', (10.2, 20.7, '1i', '2i'))
         self.checkParam(widget, 'tabs', '10.2 20.7 1i 2i',
                         expected=('10.2', '20.7', '1i', '2i'))
         self.checkParam(widget, 'tabs', '2c left 4c 6c center',
                         expected=('2c', 'left', '4c', '6c', 'center'))
         self.checkInvalidParam(widget, 'tabs', 'spam',
-                               errmsg='bad screen distance "spam"',
-                               keep_orig=tcl_version >= (8, 5))
+                               errmsg='bad screen distance "spam"')
 
-    @requires_tcl(8, 5)
     def test_configure_tabstyle(self):
         widget = self.create()
         self.checkEnumParam(widget, 'tabstyle', 'tabular', 'wordprocessor')
@@ -677,10 +655,7 @@ class TextTest(AbstractWidgetTest, unittest.TestCase):
 
     def test_configure_wrap(self):
         widget = self.create()
-        if tcl_version < (8, 5):
-            self.checkParams(widget, 'wrap', 'char', 'none', 'word')
-        else:
-            self.checkEnumParam(widget, 'wrap', 'char', 'none', 'word')
+        self.checkEnumParam(widget, 'wrap', 'char', 'none', 'word')
 
     def test_bbox(self):
         widget = self.create()
@@ -803,7 +778,7 @@ class ListboxTest(AbstractWidgetTest, unittest.TestCase):
         self.checkEnumParam(widget, 'activestyle',
                             'dotbox', 'none', 'underline')
 
-    test_justify = requires_tcl(8, 6, 5)(StandardOptionsTests.test_configure_justify)
+    test_configure_justify = requires_tcl(8, 6, 5)(StandardOptionsTests.test_configure_justify)
 
     def test_configure_listvariable(self):
         widget = self.create()
@@ -942,7 +917,7 @@ class ScaleTest(AbstractWidgetTest, unittest.TestCase):
 
     def test_configure_from(self):
         widget = self.create()
-        conv = False if get_tk_patchlevel() >= (8, 6, 10) else float_round
+        conv = float if get_tk_patchlevel() >= (8, 6, 10) else float_round
         self.checkFloatParam(widget, 'from', 100, 14.9, 15.1, conv=conv)
 
     def test_configure_label(self):
@@ -1058,12 +1033,12 @@ class PanedWindowTest(AbstractWidgetTest, unittest.TestCase):
     def test_configure_handlesize(self):
         widget = self.create()
         self.checkPixelsParam(widget, 'handlesize', 8, 9.4, 10.6, -3, '2m',
-                              conv=noconv)
+                              conv=False)
 
     def test_configure_height(self):
         widget = self.create()
         self.checkPixelsParam(widget, 'height', 100, 101.2, 102.6, -100, 0, '1i',
-                              conv=noconv)
+                              conv=False)
 
     def test_configure_opaqueresize(self):
         widget = self.create()
@@ -1079,7 +1054,7 @@ class PanedWindowTest(AbstractWidgetTest, unittest.TestCase):
         widget = self.create()
         self.checkPixelsParam(widget, 'proxyborderwidth',
                               0, 1.3, 2.9, 6, -2, '10p',
-                              conv=noconv)
+                              conv=False)
 
     @requires_tcl(8, 6, 5)
     def test_configure_proxyrelief(self):
@@ -1101,7 +1076,7 @@ class PanedWindowTest(AbstractWidgetTest, unittest.TestCase):
     def test_configure_sashwidth(self):
         widget = self.create()
         self.checkPixelsParam(widget, 'sashwidth', 10, 11.1, 15.6, -3, '1m',
-                              conv=noconv)
+                              conv=False)
 
     def test_configure_showhandle(self):
         widget = self.create()
@@ -1110,7 +1085,7 @@ class PanedWindowTest(AbstractWidgetTest, unittest.TestCase):
     def test_configure_width(self):
         widget = self.create()
         self.checkPixelsParam(widget, 'width', 402, 403.4, 404.6, -402, 0, '5i',
-                              conv=noconv)
+                              conv=False)
 
     def create2(self):
         p = self.create()
@@ -1130,15 +1105,12 @@ class PanedWindowTest(AbstractWidgetTest, unittest.TestCase):
             self.assertEqual(v, p.paneconfigure(b, k))
             self.assertEqual(v[4], p.panecget(b, k))
 
-    def check_paneconfigure(self, p, b, name, value, expected, stringify=False):
-        conv = lambda x: x
-        if not self.wantobjects or stringify:
+    def check_paneconfigure(self, p, b, name, value, expected):
+        if not self.wantobjects:
             expected = str(expected)
-        if self.wantobjects and stringify:
-            conv = str
         p.paneconfigure(b, **{name: value})
-        self.assertEqual(conv(p.paneconfigure(b, name)[4]), expected)
-        self.assertEqual(conv(p.panecget(b, name)), expected)
+        self.assertEqual(p.paneconfigure(b, name)[4], expected)
+        self.assertEqual(p.panecget(b, name), expected)
 
     def check_paneconfigure_bad(self, p, b, name, msg):
         with self.assertRaisesRegex(TclError, msg):
@@ -1158,12 +1130,10 @@ class PanedWindowTest(AbstractWidgetTest, unittest.TestCase):
 
     def test_paneconfigure_height(self):
         p, b, c = self.create2()
-        self.check_paneconfigure(p, b, 'height', 10, 10,
-                                 stringify=get_tk_patchlevel() < (8, 5, 11))
+        self.check_paneconfigure(p, b, 'height', 10, 10)
         self.check_paneconfigure_bad(p, b, 'height',
                                      'bad screen distance "badValue"')
 
-    @requires_tcl(8, 5)
     def test_paneconfigure_hide(self):
         p, b, c = self.create2()
         self.check_paneconfigure(p, b, 'hide', False, 0)
@@ -1196,7 +1166,6 @@ class PanedWindowTest(AbstractWidgetTest, unittest.TestCase):
                                      'be a string containing zero or more of '
                                      'n, e, s, and w')
 
-    @requires_tcl(8, 5)
     def test_paneconfigure_stretch(self):
         p, b, c = self.create2()
         self.check_paneconfigure(p, b, 'stretch', 'alw', 'always')
@@ -1206,8 +1175,7 @@ class PanedWindowTest(AbstractWidgetTest, unittest.TestCase):
 
     def test_paneconfigure_width(self):
         p, b, c = self.create2()
-        self.check_paneconfigure(p, b, 'width', 10, 10,
-                                 stringify=get_tk_patchlevel() < (8, 5, 11))
+        self.check_paneconfigure(p, b, 'width', 10, 10)
         self.check_paneconfigure_bad(p, b, 'width',
                                      'bad screen distance "badValue"')
 
@@ -1221,7 +1189,7 @@ class MenuTest(AbstractWidgetTest, unittest.TestCase):
         'postcommand', 'relief', 'selectcolor', 'takefocus',
         'tearoff', 'tearoffcommand', 'title', 'type',
     )
-    _conv_pixels = noconv
+    _conv_pixels = False
 
     def create(self, **kwargs):
         return tkinter.Menu(self.root, **kwargs)
@@ -1244,8 +1212,11 @@ class MenuTest(AbstractWidgetTest, unittest.TestCase):
 
     def test_configure_type(self):
         widget = self.create()
-        self.checkEnumParam(widget, 'type',
-                'normal', 'tearoff', 'menubar')
+        self.checkEnumParam(
+            widget, 'type',
+            'normal', 'tearoff', 'menubar',
+            errmsg='bad type "{}": must be normal, tearoff, or menubar',
+            )
 
     def test_entryconfigure(self):
         m1 = self.create()
@@ -1290,7 +1261,7 @@ class MessageTest(AbstractWidgetTest, unittest.TestCase):
         'justify', 'padx', 'pady', 'relief',
         'takefocus', 'text', 'textvariable', 'width',
     )
-    _conv_pad_pixels = noconv
+    _conv_pad_pixels = False
 
     def create(self, **kwargs):
         return tkinter.Message(self.root, **kwargs)
